﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Algorithms.DataAnalysis {
  internal class M5Parameters {
    private readonly ISpliter splitter;
    private readonly IPruning pruning;
    private readonly ILeafModel leafModel;
    private readonly int minLeafSize;
    private readonly IRegressionProblemData problemData;
    private readonly IRandom random;
    private readonly ResultCollection results;
    public ISpliter Spliter {
      get { return splitter; }
    }
    public IPruning Pruning {
      get { return pruning; }
    }
    public ILeafModel LeafModel {
      get { return leafModel; }
    }
    public int MinLeafSize {
      get { return minLeafSize; }
    }
    private IRegressionProblemData ProblemData {
      get { return problemData; }
    }
    public IRandom Random {
      get { return random; }
    }
    public ResultCollection Results {
      get { return results; }
    }
    public IEnumerable<string> AllowedInputVariables {
      get { return ProblemData.AllowedInputVariables; }
    }
    public string TargetVariable {
      get { return ProblemData.TargetVariable; }
    }
    public IDataset Data {
      get { return ProblemData.Dataset; }
    }

    public M5Parameters(IPruning pruning, int minleafSize, ILeafModel leafModel,
      IRegressionProblemData problemData, IRandom random, ISpliter splitter, ResultCollection results) {
      this.problemData = problemData;
      this.random = random;
      this.leafModel = leafModel;
      this.splitter = splitter;
      this.pruning = pruning;
      this.results = results;
      minLeafSize = Math.Max(pruning.MinLeafSize(problemData, leafModel), Math.Max(minleafSize, leafModel.MinLeafSize(problemData)));
    }

    public M5Parameters(ILeafModel modeltype, IRegressionProblemData problemData, IRandom random) {
      this.problemData = problemData;
      this.random = random;
      leafModel = modeltype;
    }
  }
}